import "core-js/modules/es.array.join";
import _forEachInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/for-each";
import _Object$assign from "@babel/runtime-corejs3/core-js-stable/object/assign";
import _everyInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/every";
import _Object$keys from "@babel/runtime-corejs3/core-js-stable/object/keys";
import _assertThisInitialized from "@babel/runtime-corejs3/helpers/esm/assertThisInitialized";
import _inheritsLoose from "@babel/runtime-corejs3/helpers/esm/inheritsLoose";
import _defineProperty from "@babel/runtime-corejs3/helpers/esm/defineProperty";
import Builder from './Builder';
import isObject from './isObject';

var ShapeBuilder = function (_Builder) {
  _inheritsLoose(ShapeBuilder, _Builder);

  function ShapeBuilder(contents) {
    var _this;

    _this = _Builder.call(this, 'shape', {}) || this;

    _defineProperty(_assertThisInitialized(_this), "contents", void 0);

    _defineProperty(_assertThisInitialized(_this), "isExact", false);

    if ("production" !== process.env.NODE_ENV) {
      var _context;

      _this.invariant(isObject(contents) && _Object$keys(contents).length > 0 && _everyInstanceProperty(_context = _Object$keys(contents)).call(_context, function (key) {
        return contents[key] instanceof Builder;
      }), 'A non-empty object of properties to blueprints are required for a shape.');
    }

    _this.contents = contents;
    return _this;
  }

  var _proto = ShapeBuilder.prototype;

  _proto.exact = function exact() {
    this.isExact = true;
    return this;
  };

  _proto.runChecks = function runChecks(path, initialValue, struct, options) {
    var _context2,
        _this2 = this;

    if (options === void 0) {
      options = {};
    }

    var value = {};
    var object = initialValue || this.defaultValue || {};

    if ("production" !== process.env.NODE_ENV) {
      this.invariant(isObject(object), 'Value passed to shape must be an object.', path);
    }

    var unknownFields = _Object$assign({}, object);

    _forEachInstanceProperty(_context2 = _Object$keys(this.contents)).call(_context2, function (baseKey) {
      var key = baseKey;
      var builder = _this2.contents[key];

      if (builder instanceof Builder) {
        value[key] = builder.runChecks(path + "." + key, object[key], object, options);
      } else {
        value[key] = object[key];
      }

      delete unknownFields[key];
    });

    if (this.isExact) {
      var unknownKeys = _Object$keys(unknownFields);

      if (unknownKeys.length > 0) {
        throw new Error("Unknown shape fields: " + unknownKeys.join(', ') + ".");
      }
    }

    return value;
  };

  return ShapeBuilder;
}(Builder);

export { ShapeBuilder as default };
export function shape(contents) {
  return new ShapeBuilder(contents);
}