import "core-js/modules/es.array.join";
import "core-js/modules/es.regexp.constructor";
import "core-js/modules/es.regexp.to-string";
import "core-js/modules/es.string.match";
import _everyInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/every";
import _Array$isArray from "@babel/runtime-corejs3/core-js-stable/array/is-array";
import _includesInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/includes";
import _inheritsLoose from "@babel/runtime-corejs3/helpers/esm/inheritsLoose";
import Builder from './Builder';

function isString(value) {
  return typeof value === 'string' && value !== '';
}

var StringBuilder = function (_Builder) {
  _inheritsLoose(StringBuilder, _Builder);

  function StringBuilder(defaultValue) {
    return _Builder.call(this, 'string', defaultValue || '') || this;
  }

  var _proto = StringBuilder.prototype;

  _proto.contains = function contains(token, index) {
    if (index === void 0) {
      index = 0;
    }

    if ("production" !== process.env.NODE_ENV) {
      this.invariant(isString(token), 'Contains requires a non-empty string.');
    }

    return this.addCheck(this.checkContains, token, index);
  };

  _proto.checkContains = function checkContains(path, value, token, index) {
    if (index === void 0) {
      index = 0;
    }

    if ("production" !== process.env.NODE_ENV) {
      this.invariant(_includesInstanceProperty(value).call(value, token, index), "String does not include \"" + token + "\".", path);
    }
  };

  _proto.match = function match(pattern) {
    if ("production" !== process.env.NODE_ENV) {
      this.invariant(pattern instanceof RegExp, 'Match requires a regular expression to match against.');
    }

    return this.addCheck(this.checkMatch, pattern);
  };

  _proto.checkMatch = function checkMatch(path, value, pattern) {
    if ("production" !== process.env.NODE_ENV) {
      this.invariant(!!value.match(pattern), "String does not match pattern \"" + pattern.source + "\".", path);
    }
  };

  _proto.notEmpty = function notEmpty() {
    return this.addCheck(this.checkNotEmpty);
  };

  _proto.checkNotEmpty = function checkNotEmpty(path, value) {
    if ("production" !== process.env.NODE_ENV) {
      this.invariant(isString(value), 'String cannot be empty.', path);
    }
  };

  _proto.oneOf = function oneOf(list) {
    if ("production" !== process.env.NODE_ENV) {
      this.invariant(_Array$isArray(list) && list.length > 0 && _everyInstanceProperty(list).call(list, function (item) {
        return isString(item);
      }), 'One of requires a non-empty array of strings.');
    }

    this.addCheck(this.checkOneOf, list);
    return this;
  };

  _proto.checkOneOf = function checkOneOf(path, value, list) {
    if ("production" !== process.env.NODE_ENV) {
      this.invariant(_includesInstanceProperty(list).call(list, value), "String must be one of: " + list.join(', '), path);
    }
  };

  return StringBuilder;
}(Builder);

export { StringBuilder as default };
export function string(defaultValue) {
  return new StringBuilder(defaultValue);
}