import "core-js/modules/es.array.join";
import "core-js/modules/es.function.name";
import _endsWithInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/ends-with";
import _Object$keys from "@babel/runtime-corejs3/core-js-stable/object/keys";
import _forEachInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/for-each";
import _Object$assign from "@babel/runtime-corejs3/core-js-stable/object/assign";
import Builder from './Builder';
import isObject from './isObject';
import typeOf from './typeOf';

function buildAndCheck(blueprint, struct, options, parentPath) {
  var _context;

  if (options === void 0) {
    options = {};
  }

  if (parentPath === void 0) {
    parentPath = '';
  }

  var unknownFields = _Object$assign({}, struct);

  var builtStruct = {};

  _forEachInstanceProperty(_context = _Object$keys(blueprint)).call(_context, function (baseKey) {
    var _context2;

    var key = baseKey;
    var value = struct[key];
    var builder = blueprint[key];
    var path = String(parentPath ? parentPath + "." + key : key);

    if (builder instanceof Builder || isObject(builder) && _endsWithInstanceProperty(_context2 = builder.constructor.name).call(_context2, 'Builder')) {
      builtStruct[key] = builder.runChecks(path, value, struct, options);
    } else if ("production" !== process.env.NODE_ENV) {
      throw new Error("Unknown blueprint for \"" + path + "\". Must be a builder.");
    }

    delete unknownFields[key];
  });

  if (options.unknown) {
    _Object$assign(builtStruct, unknownFields);
  } else if ("production" !== process.env.NODE_ENV) {
    var unknownKeys = _Object$keys(unknownFields);

    if (unknownKeys.length > 0) {
      throw new Error("Unknown fields: " + unknownKeys.join(', ') + ".");
    }
  }

  return builtStruct;
}

export default function optimal(struct, blueprint, options) {
  if (options === void 0) {
    options = {};
  }

  if ("production" !== process.env.NODE_ENV) {
    if (!isObject(struct)) {
      throw new TypeError("Optimal requires a plain object, found " + typeOf(struct) + ".");
    } else if (!isObject(options)) {
      throw new TypeError('Optimal options must be a plain object.');
    } else if (!isObject(blueprint)) {
      throw new TypeError('A blueprint is required.');
    }
  }

  return buildAndCheck(blueprint, struct, options);
}