"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

require("core-js/modules/es.array.join");

require("core-js/modules/es.regexp.constructor");

require("core-js/modules/es.regexp.to-string");

require("core-js/modules/es.string.match");

exports.__esModule = true;
exports.string = string;
exports.default = void 0;

var _every = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/every"));

var _isArray = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/array/is-array"));

var _includes = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/includes"));

var _inheritsLoose2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/inheritsLoose"));

var _Builder2 = _interopRequireDefault(require("./Builder"));

function isString(value) {
  return typeof value === 'string' && value !== '';
}

var StringBuilder = function (_Builder) {
  (0, _inheritsLoose2.default)(StringBuilder, _Builder);

  function StringBuilder(defaultValue) {
    return _Builder.call(this, 'string', defaultValue || '') || this;
  }

  var _proto = StringBuilder.prototype;

  _proto.contains = function contains(token, index) {
    if (index === void 0) {
      index = 0;
    }

    if ("production" !== process.env.NODE_ENV) {
      this.invariant(isString(token), 'Contains requires a non-empty string.');
    }

    return this.addCheck(this.checkContains, token, index);
  };

  _proto.checkContains = function checkContains(path, value, token, index) {
    if (index === void 0) {
      index = 0;
    }

    if ("production" !== process.env.NODE_ENV) {
      this.invariant((0, _includes.default)(value).call(value, token, index), "String does not include \"" + token + "\".", path);
    }
  };

  _proto.match = function match(pattern) {
    if ("production" !== process.env.NODE_ENV) {
      this.invariant(pattern instanceof RegExp, 'Match requires a regular expression to match against.');
    }

    return this.addCheck(this.checkMatch, pattern);
  };

  _proto.checkMatch = function checkMatch(path, value, pattern) {
    if ("production" !== process.env.NODE_ENV) {
      this.invariant(!!value.match(pattern), "String does not match pattern \"" + pattern.source + "\".", path);
    }
  };

  _proto.notEmpty = function notEmpty() {
    return this.addCheck(this.checkNotEmpty);
  };

  _proto.checkNotEmpty = function checkNotEmpty(path, value) {
    if ("production" !== process.env.NODE_ENV) {
      this.invariant(isString(value), 'String cannot be empty.', path);
    }
  };

  _proto.oneOf = function oneOf(list) {
    if ("production" !== process.env.NODE_ENV) {
      this.invariant((0, _isArray.default)(list) && list.length > 0 && (0, _every.default)(list).call(list, function (item) {
        return isString(item);
      }), 'One of requires a non-empty array of strings.');
    }

    this.addCheck(this.checkOneOf, list);
    return this;
  };

  _proto.checkOneOf = function checkOneOf(path, value, list) {
    if ("production" !== process.env.NODE_ENV) {
      this.invariant((0, _includes.default)(list).call(list, value), "String must be one of: " + list.join(', '), path);
    }
  };

  return StringBuilder;
}(_Builder2.default);

exports.default = StringBuilder;

function string(defaultValue) {
  return new StringBuilder(defaultValue);
}